﻿--------------------------------------------------------------------------------------------------------
-- Moncaí's Library
-- Author: Moncaí
-- Date: april 25th 2009
--------------------------------------------------------------------------------------------------------

local MONLIB_VERSION = 0.1;
local ML = MONCAILIB;
local _G = _G;

if (MONCAILIB and MONCAILIB.version > MONLIB_VERSION) then return; end;

MONCAILIB = {
	version = MONLIB_VERSION,
};
local ML = MONCAILIB;

--------------------------------------------------------------------------------------------------------
--
-- Utility functions
-- 
--------------------------------------------------------------------------------------------------------

ML.util = {
	RGBPercToHex = function(r, g, b)
		r = r <= 1 and r >= 0 and r or 0
		g = g <= 1 and g >= 0 and g or 0
		b = b <= 1 and b >= 0 and b or 0
		return string.format("%02x%02x%02x", r*255, g*255, b*255)
	end,

	RGBToHex = function(r, g, b)
		r = r <= 255 and r >= 0 and r or 0
		g = g <= 255 and g >= 0 and g or 0
		b = b <= 255 and b >= 0 and b or 0
		return string.format("%02x%02x%02x", r, g, b)
	end,

	-- The pipe symbol is hardcoded for optimum performance
	wrap = function(...)
		return strjoin("|", ...)
	end,

	unwrap = function(packet)
		return strsplit("|", packet)
	end,
	
};

ML.console = {
	Print = function(msg, r, g, b, frame) 
		if (not r) then r = 1.0; end
		if (not g) then g = 1.0; end
		if (not b) then b = 1.0; end
		if ( frame ) then 
			frame:AddMessage(msg,r,g,b);
		else
			if ( DEFAULT_CHAT_FRAME ) then 
				DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
			end
		end
	end,
	
	PrintEx = function(msg, col, frame) 
		Print(msg, col.r, col.g, col.b, frame);
	end,
	
	
	print_r = function(thing, ...)
		local line = ML.console.tostring_r(thing);
		for v in string.gmatch(line, "([^\n]*)\n") do
			ML.console.Print(v, ...);
		end;
	end,
};

local function tostring_r(thing, indent, done)
	indent=indent or "";
	done = done or {};
	
	if not thing then 
		return "nil";
	elseif type(thing) == "table" and not done[thing] then
		done[thing] = true;
		local result = "{ ";
		indent = indent .. "  ";
		for key, value in pairs(thing) do
			result = result .. "\n" .. indent .. tostring(key) .. " = " .. tostring_r(value, indent .. "  ", done) .. ",";
		end;
		result = result .. "}";
		return result;
	else
		return tostring(thing);
	end;
end;
ML.console.tostring_r = tostring_r;

ML.api = {};
ML.api.hooking = {
	add = function(self, fname, func) 
		self.hooks[fname] = func;
	end,
	addt = function(self, fname, obj) 
		self.hooks[fname] = obj[fname];
	end,
	adds = function(self, fname) 
		self.hooks[fname] = self.parent[fname];
	end,
	adda = function(self, obj)
		for fname, func in pairs(obj) do
			self.hooks[fname] = func;
		end;
	end,
	del = function(self, fname) 
		self.hooks[fname] = nil;
	end,	
	enable = function(self)
		for fname, func in pairs(self.hooks) do
			self.originals[fname] = _G[fname];
			_G[fname] = func;
		end;
	end,
	disable = function(self)
		for fname, func in pairs(self.originals) do
			_G[fname] = func;
		end;
	end,	
	new = function(self, caller)
		return {
			parent = caller,
			originals = {},
			hooks = {},
			add= self.add,
			adds = self.adds,
			addt = self.addt,
			adda = self.adda,
			del=self.del,
			enable = self.enable,
			disable = self.disable,
		};
	end,
}

ML.init = function(self) 
	if (self.initialized) then return; end;
	self.initialized = true;
	
	if not Print then Print = ML.console.Print; end;
	if not PrintEx then PrintEx = ML.console.PrintEx; end;
	if not print_r then print_r = ML.console.print_r; end;
end;
--------------------------------------------------------------------------------------------------------
--
-- Initialize library
-- 
--------------------------------------------------------------------------------------------------------
